@@ -356,6 +356,7 @@ static int ratbagd_new(struct ratbagd **out)
 	if (r < 0)
 		return r;
 
+	log_verbose("Initializing libratbag\n");
 	ctx->lib_ctx = ratbag_create_context(&ratbagd_lib_interface, ctx);
 	if (!ctx->lib_ctx)
 		return -ENOMEM;
@@ -520,6 +521,8 @@ static int ratbagd_run(struct ratbagd *ctx)
 			  ctx);
 	sd_event_add_post(ctx->event, NULL, before_idle_cb, ctx);
 
+	log_verbose("DBus server ready\n");
+
 	return sd_event_loop(ctx->event);
 }
 
@@ -617,12 +620,12 @@ int main(int argc, char *argv[])
 			return 0;
 		} else if (streq(argv[1], "--quiet")) {
 			log_level = LL_QUIET;
-		} else if (streq(argv[1], "--verbose=raw")) {
+		} else if (streq(argv[1], "--verbose") || streq(argv[1], "--verbose=raw")) {
 			log_level = LL_RAW;
-		} else if (streq(argv[1], "--verbose")) {
+		} else if (streq(argv[1], "--verbose=debug")) {
 			log_level = LL_VERBOSE;
 		} else {
-			fprintf(stderr, "Usage: %s [--version | --quiet | --verbose[=raw]]\n",
+			fprintf(stderr, "Usage: %s [--version | --quiet | --verbose[=debug]]\n",
 				program_invocation_short_name);
 			r = -EINVAL;
 			goto exit;
