@@ -271,8 +271,8 @@ steelseries_probe(struct ratbag_device *device)
 	struct ratbag_button *button;
 	struct ratbag_led *led;
 	int rc, button_count, led_count, device_version, mono_led, short_button;
-	_cleanup_(dpi_list_freep) struct dpi_list *dpilist = NULL;
-	_cleanup_(freep) struct dpi_range *dpirange = NULL;
+	struct dpi_list *dpilist = NULL;
+	struct dpi_range *dpirange = NULL;
 
 	unsigned int report_rates[] = { 125, 250, 500, 1000 };
 
@@ -366,10 +366,11 @@ steelseries_probe(struct ratbag_device *device)
 	drv_data = zalloc(sizeof(*drv_data));
 	ratbag_set_drv_data(device, drv_data);
 
-	steelseries_get_firmware_version(device);
-	log_debug(device->ratbag, "SteelSeries firmware version %d.%d\n",
-		  drv_data->firmware_major,
-		  drv_data->firmware_minor);
+	rc = steelseries_get_firmware_version(device);
+	if(rc == 0)
+		log_debug(device->ratbag, "SteelSeries firmware version %d.%d\n",
+			drv_data->firmware_major,
+			drv_data->firmware_minor);
 
 	steelseries_read_settings(device);
 
@@ -572,7 +573,7 @@ steelseries_write_buttons(struct ratbag_profile *profile)
 
 	msleep(10);
 	if (device_version == 3)
-		ret = ratbag_hidraw_raw_request(device, STEELSERIES_ID_BUTTONS, 
+		ret = ratbag_hidraw_raw_request(device, STEELSERIES_ID_BUTTONS,
 			buf,sizeof(buf),HID_FEATURE_REPORT,HID_REQ_SET_REPORT);
 	else
 		ret = ratbag_hidraw_output_report(device, buf, sizeof(buf));
@@ -699,8 +700,8 @@ construct_cycle_buffer(struct steelseries_led_cycle *cycle,
 
 	/* this seems to be the minimum allowed */
 	duration = max(buf[spec->point_count_idx] * 330, cycle->duration);
-	
-	hidpp_set_unaligned_le_u16(&buf[spec->duration_idx], duration);
+
+	set_unaligned_le_u16(&buf[spec->duration_idx], duration);
 }
 
 static int
@@ -709,6 +710,7 @@ steelseries_write_led_cycle(struct ratbag_led *led,
 {
 	struct ratbag_device *device = led->profile->device;
 	uint8_t buf[STEELSERIES_REPORT_SIZE] = {0};
+	int device_version = ratbag_device_data_steelseries_get_device_version(device->data);
 	int ret;
 
 	struct steelseries_led_cycle cycle;
@@ -739,7 +741,7 @@ steelseries_write_led_cycle(struct ratbag_led *led,
 
 		list_append(&cycle.points, &point[0].link);
 		break;
-	case RATBAG_LED_CYCLE:		
+	case RATBAG_LED_CYCLE:
 		point[0].color = red;
 		point[0].pos = 0x00;
 
@@ -768,7 +770,7 @@ steelseries_write_led_cycle(struct ratbag_led *led,
 
 		point[2].color = black;
 		point[2].pos = 0x7F;
-		
+
 		list_append(&cycle.points, &point[0].link);
 		list_append(&cycle.points, &point[1].link);
 		list_append(&cycle.points, &point[2].link);
@@ -782,8 +784,12 @@ steelseries_write_led_cycle(struct ratbag_led *led,
 	construct_cycle_buffer(&cycle, cycle_spec, buf, sizeof(buf));
 
 	msleep(10);
-	ret = ratbag_hidraw_raw_request(device, cycle_spec->cmd_val, buf, 
-			sizeof(buf), cycle_spec->hid_report_type, HID_REQ_SET_REPORT);
+	if (device_version == 3)
+		ret = ratbag_hidraw_raw_request(device, cycle_spec->cmd_val, buf,
+				sizeof(buf), cycle_spec->hid_report_type, HID_REQ_SET_REPORT);
+	else
+		ret = ratbag_hidraw_output_report(device, buf, sizeof(buf));
+
 	if (ret < 0)
 		return ret;
 
