@@ -833,6 +833,20 @@ hidpp10_get_current_profile(struct hidpp10_device *dev, uint8_t *current_profile
 			  "unable to find the profile at (%d,%d) in the directory\n",
 			  page, offset);
 		break;
+	case PROFILE_TYPE_FACTORY:
+		/* Factory profile is selected and profile switching is
+		 * disabled. Let's switch to the first profile because the
+		 * factory profile doesn't help anywone */
+		res = hidpp10_set_current_profile(dev, 0);
+		if (res == 0) {
+			hidpp_log_info(&dev->base, "switched from factory profile to 0\n");
+			*current_profile = 0;
+			return 0;
+		}
+
+		hidpp_log_error(&dev->base,
+				"current profile is factory profile but switching to 0 failed.\n");
+		break;
 	default:
 		hidpp_log_error(&dev->base,
 			  "Unexpected value: %02x\n",
@@ -2737,8 +2751,7 @@ hidpp10_get_device_info(struct hidpp10_device *dev)
 
 	hidpp10_get_optical_sensor_settings(dev, &reflect);
 
-	hidpp10_get_current_profile(dev, &current_profile);
-	return 0;
+	return hidpp10_get_current_profile(dev, &current_profile);
 }
 
 struct hidpp10_device*
