@@ -33,6 +33,9 @@
 
 #include "hidpp-generic.h"
 
+struct dpi_list;
+struct dpi_range;
+
 /* FIXME: that's what my G500s supports, but only pages 3-5 are valid.
  * 0 is zeroed, 1 and 2 are garbage, all above 6 is garbage */
 #define HIDPP10_NUM_PROFILES 5
@@ -57,17 +60,23 @@ struct hidpp10_dpi_mapping {
 struct hidpp10_device  {
 	struct hidpp_device base;
 	unsigned index;
+
+	bool dpi_table_is_range;
 	uint8_t dpi_count;
-	struct hidpp10_dpi_mapping *dpi_table; /* must be null terminated */
-	struct hidpp10_directory  *profile_directory; /* must be null terminated */
+	struct hidpp10_dpi_mapping *dpi_table;
 	enum hidpp10_profile_type profile_type;
 	struct hidpp10_profile *profiles;
 	unsigned int profile_count;
 };
 
 struct hidpp10_device*
-hidpp10_device_new(const struct hidpp_device *base, int idx,
-		   enum hidpp10_profile_type type);
+hidpp10_device_new(const struct hidpp_device *base,
+		   int idx,
+		   enum hidpp10_profile_type type,
+		   unsigned int profile_count);
+
+int
+hidpp10_device_read_profiles(struct hidpp10_device *dev);
 
 void
 hidpp10_device_destroy(struct hidpp10_device *dev);
@@ -84,7 +93,7 @@ hidpp10_device_destroy(struct hidpp10_device *dev);
  */
 int
 hidpp10_build_dpi_table_from_list(struct hidpp10_device *dev,
-				  const char *str_list);
+				  const struct dpi_list *list);
 
 /**
  * Builds the table of dpi for the device from the given dpi description.
@@ -99,7 +108,23 @@ hidpp10_build_dpi_table_from_list(struct hidpp10_device *dev,
  */
 int
 hidpp10_build_dpi_table_from_dpi_info(struct hidpp10_device *dev,
-				      const char *str_dpi);
+				      const struct dpi_range *range);
+
+/**
+ * Returns the lowest dpi stored in the dpi mapping table generated by
+ * hidpp10_build_dpi_table_from_list() or
+ * hidpp10_build_dpi_table_from_dpi_info()
+ */
+unsigned int
+hidpp10_dpi_table_get_min_dpi(struct hidpp10_device *dev);
+
+/**
+ * Returns the highest dpi stored in the dpi mapping table generated by
+ * hidpp10_build_dpi_table_from_list() or
+ * hidpp10_build_dpi_table_from_dpi_info()
+ */
+unsigned int
+hidpp10_dpi_table_get_max_dpi(struct hidpp10_device *dev);
 
 /* -------------------------------------------------------------------------- */
 /* 0x00: Enable HID++ Notifications                                           */
@@ -447,6 +472,8 @@ union hidpp10_macro_data {
 _Static_assert(sizeof(union hidpp10_macro_data) == 5, "Invalid size");
 
 struct hidpp10_profile {
+	uint8_t page;
+	uint8_t offset;
 	struct {
 		uint16_t xres;
 		uint16_t yres;
@@ -496,6 +523,7 @@ struct hidpp10_profile {
 	size_t num_leds;
 
 	unsigned int initialized;
+	bool enabled;
 };
 
 struct hidpp10_directory {
@@ -505,22 +533,17 @@ struct hidpp10_directory {
 } __attribute__((packed));
 
 int
-hidpp10_get_profile_directory(struct hidpp10_device *dev,
-			      struct hidpp10_directory *out,
-			      size_t nelems);
+hidpp10_get_current_profile(struct hidpp10_device *dev, uint8_t *current_profile);
 
 int
-hidpp10_get_current_profile(struct hidpp10_device *dev, int8_t *current_profile);
+hidpp10_set_current_profile(struct hidpp10_device *dev, uint16_t current_profile);
 
 int
-hidpp10_set_current_profile(struct hidpp10_device *dev, int16_t current_profile);
-
-int
-hidpp10_get_profile(struct hidpp10_device *dev, int8_t number,
+hidpp10_get_profile(struct hidpp10_device *dev, uint8_t number,
 		    struct hidpp10_profile *profile);
 
 int
-hidpp10_set_profile(struct hidpp10_device *dev, int8_t number,
+hidpp10_set_profile(struct hidpp10_device *dev, uint8_t number,
 		    struct hidpp10_profile *profile);
 
 enum ratbag_button_action_special
