@@ -48,7 +48,6 @@ hidpp20_feature_get_name(uint16_t feature)
 {
 	static char numeric[8];
 	const char *str;
-#define CASE_RETURN_STRING(a) case a: return #a; break
 
 	switch(feature)
 	{
@@ -99,11 +98,22 @@ hidpp20_sw_led_control_get_mode_string(const enum hidpp20_led_sw_ctrl_led_mode m
 		str = numeric;
 		break;
 	}
-#undef CASE_RETURN_STRING
 
 	return str;
 }
 
+const char*
+hidpp20_get_quirk_string(enum hidpp20_quirk quirk)
+{
+	switch (quirk) {
+	CASE_RETURN_STRING(HIDPP20_QUIRK_NONE);
+	CASE_RETURN_STRING(HIDPP20_QUIRK_G305);
+	CASE_RETURN_STRING(HIDPP20_QUIRK_G602);
+	}
+
+	abort();
+}
+
 static int
 hidpp20_request_command_allow_error(struct hidpp20_device *device, union hidpp20_message *msg,
 				    bool allow_error)
@@ -123,6 +133,16 @@ hidpp20_request_command_allow_error(struct hidpp20_device *device, union hidpp20
 	}
 	msg->msg.address |= DEVICE_SW_ID;
 
+	/* some mice don't support short reports */
+	if (msg->msg.report_id == REPORT_ID_SHORT && !(device->base.supported_report_types & HIDPP_REPORT_SHORT))
+		msg->msg.report_id = REPORT_ID_LONG;
+
+	/* sanity check */
+	if (msg->msg.report_id == REPORT_ID_LONG && !(device->base.supported_report_types & HIDPP_REPORT_LONG)) {
+		hidpp_log_error(&device->base, "hidpp20: trying to use unsupported report type\n");
+		return -EINVAL;
+	}
+
 	msg_len = msg->msg.report_id == REPORT_ID_SHORT ? SHORT_MESSAGE_LENGTH : LONG_MESSAGE_LENGTH;
 
 	/* Send the message to the Device */
@@ -848,6 +868,8 @@ err:
 #define CMD_COLOR_LED_EFFECTS_GET_INFO 0x00
 #define CMD_COLOR_LED_EFFECTS_GET_ZONE_INFO 0x10
 #define CMD_COLOR_LED_EFFECTS_GET_ZONE_EFFECT_INFO 0x20
+#define CMD_COLOR_LED_EFFECTS_SET_ZONE_EFFECT 0x30
+#define CMD_COLOR_LED_EFFECTS_GET_ZONE_EFFECT 0xe0
 
 int
 hidpp20_color_led_effects_get_info(struct hidpp20_device *device,
@@ -873,6 +895,7 @@ hidpp20_color_led_effects_get_info(struct hidpp20_device *device,
 		return rc;
 
 	*info = *(struct hidpp20_color_led_info *)msg.msg.parameters;
+	device->led_ext_caps = info->ext_caps;
 
 	return 0;
 }
@@ -992,6 +1015,80 @@ hidpp20_color_led_effect_get_zone_effect_info(struct hidpp20_device *device,
 	return 0;
 }
 
+int
+hidpp20_color_led_effects_set_zone_effect(struct hidpp20_device *device,
+					  uint8_t zone_index,
+					  struct hidpp20_led led)
+{
+	uint8_t feature_index;
+	union hidpp20_message msg = {
+		.msg.report_id = REPORT_ID_SHORT,
+		.msg.address = CMD_COLOR_LED_EFFECTS_SET_ZONE_EFFECT,
+		.msg.device_idx = device->index,
+		.msg.parameters[0] = zone_index,
+		.msg.parameters[12] = 1, /* write to RAM and flash */
+	};
+	int rc;
+	struct hidpp20_internal_led *internal_led = (struct hidpp20_internal_led*) &msg.msg.parameters[1];
+
+	hidpp20_onboard_profiles_write_led(internal_led, &led);
+
+	feature_index = hidpp_root_get_feature_idx(device,
+						   HIDPP_PAGE_COLOR_LED_EFFECTS);
+	if (feature_index == 0)
+		return -ENOTSUP;
+
+	msg.msg.sub_id = feature_index;
+
+	rc = hidpp20_request_command(device, &msg);
+	if (rc)
+		return rc;
+
+	return 0;
+}
+
+int
+hidpp20_color_led_effects_get_zone_effect(struct hidpp20_device *device,
+					  uint8_t zone_index,
+					  struct hidpp20_led *led)
+{
+	uint8_t feature_index;
+	union hidpp20_message msg = {
+		.msg.report_id = REPORT_ID_SHORT,
+		.msg.address = CMD_COLOR_LED_EFFECTS_GET_ZONE_EFFECT,
+		.msg.device_idx = device->index,
+		.msg.parameters[0] = zone_index,
+	};
+	struct hidpp20_internal_led *internal_led;
+	int rc;
+
+	/* hidpp20_color_led_effects_get_info() must be called first to set the capabilities */
+	if (!(device->led_ext_caps & HIDPP20_COLOR_LED_INFO_EXT_CAP_HAS_ZONE_EFFECT))
+		return -ENOTSUP;
+
+	feature_index = hidpp_root_get_feature_idx(device,
+						   HIDPP_PAGE_COLOR_LED_EFFECTS);
+	if (feature_index == 0)
+		return -ENOTSUP;
+
+	msg.msg.sub_id = feature_index;
+
+	rc = hidpp20_request_command(device, &msg);
+	if (rc)
+		return rc;
+
+	if (msg.msg.parameters[0] != zone_index)
+		return -EPROTO;
+
+	internal_led = (struct hidpp20_internal_led*) &msg.msg.parameters[1];
+
+	hidpp20_onboard_profiles_read_led(led, *internal_led);
+
+	hidpp_log_debug(&device->base, "zone %u has effect %u\n", zone_index, led->mode);
+
+	return 0;
+}
+
 /* -------------------------------------------------------------------------- */
 /* 0x8071: RGB Effects                                                        */
 /* -------------------------------------------------------------------------- */
@@ -1073,8 +1170,8 @@ hidpp20_rgb_effects_get_cluster_info(struct hidpp20_device *device,
 	info->effect_index = msg.msg.parameters[1];
 
 	info->location = get_unaligned_be_u16(&msg.msg.parameters[2]);
-	info->num_effects = get_unaligned_be_u16(&msg.msg.parameters[4]);
-	info->persistency_caps = get_unaligned_be_u16(&msg.msg.parameters[6]);
+	info->num_effects = msg.msg.parameters[4];
+	info->persistency_caps = msg.msg.parameters[5];
 
 	return 0;
 }
@@ -1432,6 +1529,11 @@ hidpp20_adjustable_dpi_get_dpi_list(struct hidpp20_device *device,
 		.msg.parameters[0] = sensor->index,
 	};
 
+	if (device->quirk == HIDPP20_QUIRK_G602) {
+		msg.msg.parameters[0] = 1;
+		i = 0;
+	}
+
 	rc = hidpp20_request_command(device, &msg);
 	if (rc)
 		return rc;
@@ -1443,6 +1545,9 @@ hidpp20_adjustable_dpi_get_dpi_list(struct hidpp20_device *device,
 	       get_unaligned_be_u16(&msg.msg.parameters[i]) != 0) {
 		uint16_t value = get_unaligned_be_u16(&msg.msg.parameters[i]);
 
+		if (device->quirk == HIDPP20_QUIRK_G602 && i == 2)
+			value += 0xe000;
+
 		if (value > 0xe000) {
 			sensor->dpi_steps = value - 0xe000;
 		} else {
@@ -1472,6 +1577,9 @@ hidpp20_adjustable_dpi_get_dpi(struct hidpp20_device *device,
 		.msg.parameters[0] = sensor->index,
 	};
 
+	if (device->quirk == HIDPP20_QUIRK_G602)
+		msg.msg.parameters[0] = 1;
+
 	rc = hidpp20_request_command(device, &msg);
 	if (rc)
 		return rc;
@@ -1551,6 +1659,9 @@ int hidpp20_adjustable_dpi_set_sensor_dpi(struct hidpp20_device *device,
 		.msg.parameters[0] = sensor->index,
 	};
 
+	if (device->quirk == HIDPP20_QUIRK_G602)
+		msg.msg.parameters[0] = 1;
+
 	set_unaligned_be_u16(&msg.msg.parameters[1], dpi);
 
 	feature_index = hidpp_root_get_feature_idx(device,
@@ -1578,6 +1689,8 @@ int hidpp20_adjustable_dpi_set_sensor_dpi(struct hidpp20_device *device,
 /* -------------------------------------------------------------------------- */
 
 #define CMD_ADJUSTABLE_REPORT_RATE_GET_REPORT_RATE_LIST 0x00
+#define CMD_ADJUSTABLE_REPORT_RATE_GET_REPORT_RATE	0x10
+#define CMD_ADJUSTABLE_REPORT_RATE_SET_REPORT_RATE	0x20
 
 int hidpp20_adjustable_report_rate_get_report_rate_list(struct hidpp20_device *device,
 							uint8_t *bitflags_ms)
@@ -1607,6 +1720,60 @@ int hidpp20_adjustable_report_rate_get_report_rate_list(struct hidpp20_device *d
 	return 0;
 }
 
+int hidpp20_adjustable_report_rate_get_report_rate(struct hidpp20_device *device,
+						   uint8_t *rate_ms)
+{
+	uint8_t feature_index;
+	int rc;
+	union hidpp20_message msg = {
+		.msg.report_id = REPORT_ID_SHORT,
+		.msg.device_idx = device->index,
+		.msg.address = CMD_ADJUSTABLE_REPORT_RATE_GET_REPORT_RATE,
+		.msg.parameters[0] = 0,
+	};
+
+	feature_index = hidpp_root_get_feature_idx(device,
+						   HIDPP_PAGE_ADJUSTABLE_REPORT_RATE);
+	if (feature_index == 0)
+		return -ENOTSUP;
+
+	msg.msg.sub_id = feature_index;
+
+	rc = hidpp20_request_command(device, &msg);
+	if (rc)
+		return rc;
+
+	*rate_ms = msg.msg.parameters[0];
+
+	return 0;
+}
+
+int hidpp20_adjustable_report_rate_set_report_rate(struct hidpp20_device *device,
+						   uint8_t rate_ms)
+{
+	uint8_t feature_index;
+	int rc;
+	union hidpp20_message msg = {
+		.msg.report_id = REPORT_ID_SHORT,
+		.msg.device_idx = device->index,
+		.msg.address = CMD_ADJUSTABLE_REPORT_RATE_SET_REPORT_RATE,
+		.msg.parameters[0] = rate_ms,
+	};
+
+	feature_index = hidpp_root_get_feature_idx(device,
+						   HIDPP_PAGE_ADJUSTABLE_REPORT_RATE);
+	if (feature_index == 0)
+		return -ENOTSUP;
+
+	msg.msg.sub_id = feature_index;
+
+	rc = hidpp20_request_command(device, &msg);
+	if (rc)
+		return rc;
+
+	return 0;
+}
+
 /* -------------------------------------------------------------------------- */
 /* 0x8100 - Onboard Profiles                                                  */
 /* -------------------------------------------------------------------------- */
@@ -1634,7 +1801,7 @@ int hidpp20_adjustable_report_rate_get_report_rate_list(struct hidpp20_device *d
 #define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G402	0x01
 #define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G303	0x02
 #define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G900	0x03
-#define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G903_HERO	0x04
+#define HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G915	0x04
 #define HIDPP20_ONBOARD_PROFILES_MACRO_TYPE_G402	0x01
 
 #define HIDPP20_USER_PROFILES_G402			0x0000
@@ -2054,7 +2221,7 @@ hidpp20_onboard_profiles_validate(struct hidpp20_device *device,
 	if ((info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G402) &&
 	    (info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G303) &&
 	    (info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G900) &&
-	    (info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G903_HERO)) {
+	    (info->profile_format_id != HIDPP20_ONBOARD_PROFILES_PROFILE_TYPE_G915)) {
 		hidpp_log_error(&device->base,
 				"Profile layout not supported: 0x%02x.\n",
 				info->profile_format_id);
@@ -2354,7 +2521,7 @@ hidpp20_onboard_profiles_write_dict(struct hidpp20_device *device,
 
 	memset(data + buffer_index, 0xff, sector_size - buffer_index);
 
-	hidpp_log_buf_debug(&device->base,
+	hidpp_log_buf_raw(&device->base,
 			   "dictionary: ",
 			   data,
 			   hidpp20_onboard_profiles_compute_dict_size(device,
@@ -2365,6 +2532,9 @@ hidpp20_onboard_profiles_write_dict(struct hidpp20_device *device,
 						   sector_size,
 						   data,
 						   true);
+	if (rc)
+		hidpp_log_error(&device->base, "failed to write profile dictionary\n");
+
 	return rc;
 }
 
@@ -2402,6 +2572,7 @@ hidpp20_buttons_to_cpu(struct hidpp20_device *device,
 			break;
 		case HIDPP20_BUTTON_SPECIAL:
 			button->special.special = b->special.special;
+			button->special.profile = b->special.profile;
 			break;
 		case HIDPP20_BUTTON_MACRO:
 			if (profile->macros[i]) {
@@ -2461,6 +2632,7 @@ hidpp20_buttons_from_cpu(struct hidpp20_profile *profile,
 			break;
 		case HIDPP20_BUTTON_SPECIAL:
 			button->special.special = b->special.special;
+			button->special.profile = b->special.profile;
 			break;
 		case HIDPP20_BUTTON_DISABLED:
 			break;
@@ -2476,7 +2648,7 @@ hidpp20_buttons_from_cpu(struct hidpp20_profile *profile,
 	}
 }
 
-static void
+void
 hidpp20_onboard_profiles_read_led(struct hidpp20_led *led,
 				  struct hidpp20_internal_led internal_led)
 {
@@ -2492,6 +2664,10 @@ hidpp20_onboard_profiles_read_led(struct hidpp20_led *led,
 		if (brightness == 0)
 			brightness = 100;
 		break;
+	case HIDPP20_LED_STARLIGHT:
+		led->color = internal_led.effect.starlight.color_sky;
+		led->extra_color = internal_led.effect.starlight.color_star;
+		break;
 	case HIDPP20_LED_BREATHING:
 		period = hidpp_be_u16_to_cpu(internal_led.effect.breath.period_or_speed);
 		brightness = internal_led.effect.breath.intensity;
@@ -2499,11 +2675,18 @@ hidpp20_onboard_profiles_read_led(struct hidpp20_led *led,
 			brightness = 100;
 		led->color = internal_led.effect.breath.color;
 		break;
+	case HIDPP20_LED_RIPPLE:
+		period = hidpp_be_u16_to_cpu(internal_led.effect.breath.period_or_speed);
+		led->color = internal_led.effect.ripple.color;
+		break;
 	case HIDPP20_LED_ON:
 		led->color = internal_led.effect.fixed.color;
 		break;
 	case HIDPP20_LED_OFF:
 		break;
+	default:
+		memcpy(led->original, &internal_led, sizeof(internal_led));
+		break;
 	}
 
 	led->period = period;
@@ -2602,6 +2785,15 @@ hidpp20_onboard_profiles_initialize(struct hidpp20_device *device,
 						  HIDPP20_USER_PROFILES_G402,
 						  profiles->sector_size,
 						  data);
+
+	if (rc && device->quirk == HIDPP20_QUIRK_G305) {
+		/* The G305 has a bug where it throws an ERR_INVALID_ARGUMENT
+		   if the sector has not been written to yet. If this happens
+		   we will read the ROM profiles.*/
+		read_userdata = false;
+		goto read_profiles;
+	}
+
 	if (rc < 0)
 		return rc; // ignore_clang_sa_mem_leak
 
@@ -2635,6 +2827,7 @@ hidpp20_onboard_profiles_initialize(struct hidpp20_device *device,
 		read_userdata = false;
 	}
 
+read_profiles:
 	for (i = 0; i < profiles->num_profiles; i++) {
 		if (read_userdata) {
 			hidpp_log_debug(&device->base, "Parsing profile %u\n", i);
@@ -2669,7 +2862,7 @@ hidpp20_onboard_profiles_initialize(struct hidpp20_device *device,
 	return profiles->num_profiles;
 }
 
-static void
+void
 hidpp20_onboard_profiles_write_led(struct hidpp20_internal_led *internal_led,
 				   struct hidpp20_led *led)
 {
@@ -2688,24 +2881,31 @@ hidpp20_onboard_profiles_write_led(struct hidpp20_internal_led *internal_led,
 		else
 			internal_led->effect.cycle.intensity = 0;
 		break;
+	case HIDPP20_LED_STARLIGHT:
+		internal_led->effect.starlight.color_sky = led->color;
+		internal_led->effect.starlight.color_star = led->extra_color;
+		break;
 	case HIDPP20_LED_BREATHING:
-		internal_led->effect.breath.color.red = led->color.red;
-		internal_led->effect.breath.color.blue = led->color.blue;
-		internal_led->effect.breath.color.green = led->color.green;
+		internal_led->effect.breath.color = led->color;
 		internal_led->effect.breath.period_or_speed = hidpp_cpu_to_be_u16(period);
 		if (brightness < 100)
 			internal_led->effect.breath.intensity = brightness;
 		else
 			internal_led->effect.breath.intensity = 0;
 		break;
+	case HIDPP20_LED_RIPPLE:
+		internal_led->effect.ripple.color = led->color;
+		internal_led->effect.ripple.period = hidpp_cpu_to_be_u16(period);
+		break;
 	case HIDPP20_LED_ON:
-		internal_led->effect.fixed.color.red = led->color.red;
-		internal_led->effect.fixed.color.blue = led->color.blue;
-		internal_led->effect.fixed.color.green = led->color.green;
+		internal_led->effect.fixed.color = led->color;
 		internal_led->effect.fixed.effect = 0;
 		break;
 	case HIDPP20_LED_OFF:
 		break;
+	default:
+		memcpy(internal_led, led->original, sizeof(*internal_led));
+		break;
 	}
 }
 
@@ -2753,8 +2953,10 @@ hidpp20_onboard_profiles_write_profile(struct hidpp20_device *device,
 	memcpy(pdata->profile.name.txt, profile->name, sizeof(profile->name));
 
 	rc = hidpp20_onboard_profiles_write_sector(device, sector, sector_size, data, true);
-	if (rc < 0)
+	if (rc < 0) {
+		hidpp_log_error(&device->base, "failed to write profile\n");
 		return rc;
+	}
 
 	return 0;
 }
@@ -2837,7 +3039,7 @@ hidpp20_onboard_profiles_get_code_from_special(enum ratbag_button_action_special
 /* -------------------------------------------------------------------------- */
 
 struct hidpp20_device *
-hidpp20_device_new(const struct hidpp_device *base, unsigned int idx)
+hidpp20_device_new(const struct hidpp_device *base, unsigned int idx, struct hidpp_hid_report *reports, unsigned int num_reports)
 {
 	struct hidpp20_device *dev;
 	int rc;
@@ -2850,6 +3052,14 @@ hidpp20_device_new(const struct hidpp_device *base, unsigned int idx)
 	dev->proto_major = 1;
 	dev->proto_minor = 0;
 
+	dev->led_ext_caps = 0;
+
+	hidpp_get_supported_report_types(&(dev->base), reports, num_reports);
+
+	if (!(dev->base.supported_report_types & HIDPP_REPORT_SHORT) &&
+	    !(dev->base.supported_report_types & HIDPP_REPORT_LONG))
+	    goto err;
+
 	rc = hidpp20_root_get_protocol_version(dev, &dev->proto_major, &dev->proto_minor);
 	if (rc) {
 		/* communication error, best to ignore the device */
