@@ -51,6 +51,12 @@ struct hidpp20_feature {
 	uint8_t type;
 };
 
+enum hidpp20_quirk {
+	HIDPP20_QUIRK_NONE,
+	HIDPP20_QUIRK_G305,
+	HIDPP20_QUIRK_G602,
+};
+
 struct hidpp20_device {
 	struct hidpp_device base;
 	unsigned int index;
@@ -58,18 +64,25 @@ struct hidpp20_device {
 	unsigned proto_minor;
 	unsigned feature_count;
 	struct hidpp20_feature *feature_list;
+	enum hidpp20_quirk quirk;
+	unsigned int led_ext_caps;
 };
 
 int hidpp20_request_command(struct hidpp20_device *dev, union hidpp20_message *msg);
 
+#define CASE_RETURN_STRING(a) case a: return #a; break
+
 const char *hidpp20_feature_get_name(uint16_t feature);
 
+const char *hidpp20_get_quirk_string(enum hidpp20_quirk quirk);
+
 /* -------------------------------------------------------------------------- */
 /* generic hidpp20 device operations                                          */
 /* -------------------------------------------------------------------------- */
 
 struct hidpp20_device *
-hidpp20_device_new(const struct hidpp_device *base, unsigned int idx);
+hidpp20_device_new(const struct hidpp_device *base, unsigned int idx,
+		   struct hidpp_hid_report *reports, unsigned int num_reports);
 
 void
 hidpp20_device_destroy(struct hidpp20_device *device);
@@ -420,6 +433,12 @@ int hidpp20_adjustable_dpi_set_sensor_dpi(struct hidpp20_device *device,
 int hidpp20_adjustable_report_rate_get_report_rate_list(struct hidpp20_device *device,
 							uint8_t *bitflags_ms);
 
+int hidpp20_adjustable_report_rate_get_report_rate(struct hidpp20_device *device,
+						   uint8_t *rate_ms);
+
+int hidpp20_adjustable_report_rate_set_report_rate(struct hidpp20_device *device,
+						   uint8_t rate_ms);
+
 /* -------------------------------------------------------------------------- */
 /* 0x8070v4 - Color LED effects                                               */
 /* -------------------------------------------------------------------------- */
@@ -483,6 +502,18 @@ hidpp20_rgb_effects_get_effect_info(struct hidpp20_device *device,
 				    uint8_t effect_index,
 				    struct hidpp20_rgb_effect_info *info);
 
+struct hidpp20_led;
+
+int
+hidpp20_color_led_effects_set_zone_effect(struct hidpp20_device *device,
+					  uint8_t zone_index,
+					  struct hidpp20_led led);
+
+int
+hidpp20_color_led_effects_get_zone_effect(struct hidpp20_device *device,
+					  uint8_t zone_index,
+					  struct hidpp20_led *led);
+
 enum hidpp20_color_led_location {
 	HIDPP20_COLOR_LED_LOCATION_UNDEFINED = 0,
 	HIDPP20_COLOR_LED_LOCATION_PRIMARY,
@@ -587,6 +618,8 @@ union hidpp20_button_binding {
 	struct {
 		uint8_t type; /* HIDPP20_BUTTON_SPECIAL */
 		uint8_t special;
+		uint8_t reserved;
+		uint8_t profile;
 	} __attribute__((packed)) special;
 	struct {
 		uint8_t type; /* HIDPP20_BUTTON_MACRO */
@@ -686,7 +719,11 @@ enum hidpp20_led_mode {
 	HIDPP20_LED_OFF = 0x00,
 	HIDPP20_LED_ON = 0x01,
 	HIDPP20_LED_CYCLE = 0x03,
+	HIDPP20_LED_COLOR_WAVE = 0x04,
+	HIDPP20_LED_STARLIGHT = 0x05,
 	HIDPP20_LED_BREATHING = 0x0a,
+	HIDPP20_LED_RIPPLE = 0x0b,
+	HIDPP20_LED_CUSTOM = 0x0c,
 };
 
 enum hidpp20_led_waveform {
@@ -711,27 +748,48 @@ struct hidpp20_internal_led {
 			uint16_t period_or_speed; /* period in ms, speed is device dependent */
 			uint8_t intensity; /* 1 - 100 percent, 0 means 100 */
 		} __attribute__((packed)) cycle;
+		struct hidpp20_led_starlight {
+			struct hidpp20_color color_sky;
+			struct hidpp20_color color_star;
+		} __attribute__((packed)) starlight;
 		struct hidpp20_led_breath {
 			struct hidpp20_color color;
 			uint16_t period_or_speed; /* period in ms, speed is device dependent */
 			uint8_t waveform; /* enum hidpp20_led_waveform */
 			uint8_t intensity; /* 1 - 100 percent, 0 means 100 */
 		} __attribute__((packed)) breath;
+		struct hidpp20_led_ripple {
+			struct hidpp20_color color;
+			uint8_t reserved;
+			uint16_t period;
+		} __attribute__((packed)) ripple;
+		struct hidpp20_led_custom {
+			uint8_t slot;
+			uint16_t init_frame;
+			uint16_t lenght;
+			uint16_t frame_period;
+			uint8_t intensity;
+		} __attribute__((packed)) custom;
 		uint8_t padding[10];
 	} __attribute__((packed)) effect;
 };
 _Static_assert(sizeof(struct hidpp20_led_fixed) == 4, "Invalid size");
 _Static_assert(sizeof(struct hidpp20_led_cycle) == 8, "Invalid size");
+_Static_assert(sizeof(struct hidpp20_led_starlight) == 6, "Invalid size");
 _Static_assert(sizeof(struct hidpp20_led_breath) == 7, "Invalid size");
 _Static_assert(sizeof(struct hidpp20_internal_led) == 11, "Invalid size");
+_Static_assert(sizeof(struct hidpp20_led_ripple) == 6, "Invalid size");
+_Static_assert(sizeof(struct hidpp20_led_custom) == 8, "Invalid size");
 
 typedef uint8_t percent_t;
 
 struct hidpp20_led {
 	enum hidpp20_led_mode mode;
 	struct hidpp20_color color;
+	struct hidpp20_color extra_color;
 	uint16_t period;
 	percent_t brightness;
+	uint8_t original[sizeof(struct hidpp20_internal_led)];
 };
 
 #define HIDPP20_MACRO_NOOP			0x01
@@ -907,6 +965,14 @@ hidpp20_onboard_profiles_allocate_sector(struct hidpp20_profiles *profiles)
 	return zalloc(profiles->sector_size);
 }
 
+void
+hidpp20_onboard_profiles_read_led(struct hidpp20_led *led,
+				  struct hidpp20_internal_led internal_led);
+
+void
+hidpp20_onboard_profiles_write_led(struct hidpp20_internal_led *internal_led,
+				   struct hidpp20_led *led);
+
 /* -------------------------------------------------------------------------- */
 /* 0x8110 - Mouse Button Spy                                                  */
 /* -------------------------------------------------------------------------- */
