@@ -33,6 +33,7 @@
 #include "libratbag.h"
 #include "libratbag-private.h"
 #include "libratbag-data.h"
+#include "hidpp20.h"
 
 #define GROUP_DEVICE "Device"
 
@@ -54,6 +55,7 @@ enum driver {
 
 struct data_hidpp20 {
 	int index;
+	enum hidpp20_quirk quirk;
 };
 
 struct data_hidpp10 {
@@ -152,6 +154,7 @@ init_data_hidpp20(struct ratbag *ratbag,
 	const char *group = "Driver/hidpp20";
 	GError *error = NULL;
 	int num;
+	char *str;
 
 	data->hidpp20.index = -1;
 
@@ -160,6 +163,15 @@ init_data_hidpp20(struct ratbag *ratbag,
 		data->hidpp20.index = num;
 	if (error)
 		g_error_free(error);
+
+	str = g_key_file_get_string(keyfile, group, "Quirk", NULL);
+	data->hidpp20.quirk = HIDPP20_QUIRK_NONE;
+	if (str) {
+		if (streq(str, "G305"))
+			data->hidpp20.quirk = HIDPP20_QUIRK_G305;
+		else if(streq(str, "G602"))
+			data->hidpp20.quirk = HIDPP20_QUIRK_G602;
+	}
 }
 
 static void
@@ -334,6 +346,8 @@ parse_ledtypes(char **strv, enum ratbag_led_type *types, size_t ntypes)
 			types[i] = RATBAG_LED_TYPE_SIDE;
 		else if (streq(s, "wheel"))
 			types[i] = RATBAG_LED_TYPE_WHEEL;
+		else if (streq(s, "switches"))
+			types[i] = RATBAG_LED_TYPE_SWITCHES;
 		else
 			return -1;
 
@@ -562,6 +576,14 @@ ratbag_device_data_hidpp20_get_index(const struct ratbag_device_data *data)
 	return data->hidpp20.index;
 }
 
+enum hidpp20_quirk
+ratbag_device_data_hidpp20_get_quirk(const struct ratbag_device_data *data)
+{
+	assert(data->drivertype == HIDPP20);
+
+	return data->hidpp20.quirk;
+}
+
 /* SteelSeries */
 
 int
