@@ -1239,6 +1239,8 @@ ratbag_hidraw_parse_report_descriptor(struct ratbag_device *device)
 	if (rc < 0)
 		return rc;
 
+	log_debug(device->ratbag, "Parsing HID report descriptor\n");
+
 	i = 0;
 	usage_page = 0;
 	usage = 0;
@@ -1260,7 +1262,7 @@ ratbag_hidraw_parse_report_descriptor(struct ratbag_device *device)
 		switch (hid) {
 		case HID_REPORT_ID:
 			if (hidraw->reports) {
-				log_debug(device->ratbag, "report ID %02x\n", content);
+				log_debug(device->ratbag, "- HID report ID %02x\n", content);
 				hidraw->reports[hidraw->num_reports].report_id = content;
 				hidraw->reports[hidraw->num_reports].usage_page = usage_page;
 				hidraw->reports[hidraw->num_reports].usage = usage;
@@ -1328,6 +1330,10 @@ ratbag_open_hidraw_node(struct ratbag_device *device, struct udev_device *hidraw
 		goto err;
 	}
 
+	log_debug(device->ratbag,
+		  "hidraw info: bus %#04x vendor %#04x product %#04x\n",
+		  info.bustype, info.vendor, info.product);
+
 	/* check basic matching between the hidraw node and the ratbag device */
 	if (info.bustype != device->ids.bustype ||
 	    (info.vendor & 0xFFFF )!= device->ids.vendor ||
@@ -1582,6 +1588,23 @@ ratbag_hidraw_raw_request(struct ratbag_device *device, unsigned char reportnum,
 	return -EINVAL;
 }
 
+int
+ratbag_hidraw_get_feature_report(struct ratbag_device *device, unsigned char reportnum,
+				 uint8_t *buf, size_t len)
+{
+	return ratbag_hidraw_raw_request(device, reportnum, buf, len,
+					 HID_FEATURE_REPORT, HID_REQ_GET_REPORT);
+}
+
+int
+ratbag_hidraw_set_feature_report(struct ratbag_device *device, unsigned char reportnum,
+				 uint8_t *buf, size_t len)
+{
+	return ratbag_hidraw_raw_request(device, reportnum, buf, len,
+					 HID_FEATURE_REPORT, HID_REQ_SET_REPORT);
+}
+
+
 int
 ratbag_hidraw_output_report(struct ratbag_device *device, uint8_t *buf, size_t len)
 {
