@@ -328,3 +328,35 @@ err:
 	dpi_list_free(list);
 	return NULL;
 }
+
+static inline uint16_t
+get_unaligned_be_u16(uint8_t *buf)
+{
+	return (buf[0] << 8) | buf[1];
+}
+
+static inline void
+set_unaligned_be_u16(uint8_t *buf, uint16_t value)
+{
+	buf[0] = value >> 8;
+	buf[1] = value & 0xFF;
+}
+
+static inline uint16_t
+get_unaligned_le_u16(uint8_t *buf)
+{
+	return (buf[1] << 8) | buf[0];
+}
+
+static inline void
+set_unaligned_le_u16(uint8_t *buf, uint16_t value)
+{
+	buf[0] = value & 0xFF;
+	buf[1] = value >> 8;
+}
+
+static inline uint32_t
+get_unaligned_be_u32(uint8_t *buf)
+{
+	return (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
+}
