@@ -71,17 +71,29 @@ ratbag_default_log_func(struct ratbag *ratbag,
 			const char *format, va_list args)
 {
 	const char *prefix;
+	FILE *out = stdout;
 
 	switch(priority) {
-	case RATBAG_LOG_PRIORITY_RAW: prefix = "raw"; break;
-	case RATBAG_LOG_PRIORITY_DEBUG: prefix = "debug"; break;
-	case RATBAG_LOG_PRIORITY_INFO: prefix = "info"; break;
-	case RATBAG_LOG_PRIORITY_ERROR: prefix = "error"; break;
-	default: prefix="<invalid priority>"; break;
+	case RATBAG_LOG_PRIORITY_RAW:
+		prefix = "raw";
+		break;
+	case RATBAG_LOG_PRIORITY_DEBUG:
+		prefix = "debug";
+		break;
+	case RATBAG_LOG_PRIORITY_INFO:
+		prefix = "info";
+		break;
+	case RATBAG_LOG_PRIORITY_ERROR:
+		prefix = "error";
+		out = stderr;
+		break;
+	default:
+		prefix="<invalid priority>";
+		break;
 	}
 
-	fprintf(stderr, "ratbag %s: ", prefix);
-	vfprintf(stderr, format, args);
+	fprintf(out, "ratbag %s: ", prefix);
+	vfprintf(out, format, args);
 }
 
 void
@@ -256,7 +268,7 @@ ratbag_sanity_check_device(struct ratbag_device *device)
 		}
 
 		nres = ratbag_profile_get_num_resolutions(profile);
-		if (nres == 0 || nres > 16) {
+		if (nres > 16) {
 				log_bug_libratbag(ratbag,
 						  "%s: invalid number of resolutions (%d)\n",
 						  device->name,
@@ -322,7 +334,7 @@ out:
 }
 
 static inline bool
-ratbag_test_driver(struct ratbag_device *device,
+ratbag_try_driver(struct ratbag_device *device,
 		   const struct input_id *dev_id,
 		   const char *driver_name,
 		   const struct ratbag_test_device *test_device)
@@ -377,9 +389,9 @@ ratbag_driver_fallback_logitech(struct ratbag_device *device,
 	if (dev_id->vendor != USB_VENDOR_ID_LOGITECH)
 		return false;
 
-	rc = ratbag_test_driver(device, dev_id, "hidpp20", NULL);
+	rc = ratbag_try_driver(device, dev_id, "hidpp20", NULL);
 	if (!rc)
-		rc = ratbag_test_driver(device, dev_id, "hidpp10", NULL);
+		rc = ratbag_try_driver(device, dev_id, "hidpp10", NULL);
 
 	return rc;
 }
@@ -401,7 +413,7 @@ ratbag_assign_driver(struct ratbag_device *device,
 
 	if (driver_name) {
 		log_debug(device->ratbag, "device assigned driver %s\n", driver_name);
-		rc = ratbag_test_driver(device, dev_id, driver_name, test_device);
+		rc = ratbag_try_driver(device, dev_id, driver_name, test_device);
 	} else {
 		rc = ratbag_driver_fallback_logitech(device, dev_id);
 	}
@@ -937,6 +949,8 @@ ratbag_device_commit(struct ratbag_device *device)
 	list_for_each(profile, &device->profiles, link) {
 		profile->dirty = false;
 
+		profile->rate_dirty = false;
+
 		list_for_each(button, &profile->buttons, link)
 			button->dirty = false;
 
@@ -1105,6 +1119,7 @@ ratbag_profile_set_report_rate(struct ratbag_profile *profile,
 	if (profile->hz != hz) {
 		profile->hz = hz;
 		profile->dirty = true;
+		profile->rate_dirty = true;
 	}
 
 	return RATBAG_SUCCESS;
