@@ -24,6 +24,7 @@
 #pragma once
 
 #include <stdlib.h>
+#include <stdint.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <libudev.h>
@@ -95,8 +96,7 @@ struct ratbag_device;
  * @ingroup profile
  * @struct ratbag_profile
  *
- * A handle to a profile context on devices with the @ref
- * RATBAG_DEVICE_CAP_SWITCHABLE_PROFILE capability.
+ * A handle to a profile context on devices.
  * This struct is refcounted, use ratbag_profile_ref() and
  * ratbag_profile_unref().
  */
@@ -433,28 +433,43 @@ ratbag_device_get_name(const struct ratbag_device* device);
 /**
  * @ingroup device
  *
- * @param device A previously initialized ratbag device
- * @return The file name of the svg drawing associated with the given ratbag.
- * If there is no file associated to the device, NULL is returned.
+ * Returns the bustype of the device, "usb", "bluetooth" or NULL for
+ * unknown.
  */
 const char *
-ratbag_device_get_svg_name(const struct ratbag_device* device);
+ratbag_device_get_bustype(const struct ratbag_device *device);
 
 /**
  * @ingroup device
  *
- * Note that a device may not support any of the capabilities but still
- * initialize fine otherwise. This is the case for devices that have no
- * configurable options set, or for devices that have some configuration
- * options but none that are currently exposed by libratbag. A client is
- * expected to handle this situation.
- *
- * @retval 1 The device has the capability
- * @retval 0 The device does not have the capability
+ * Returns the vendor ID of the device.
  */
-bool
-ratbag_device_has_capability(const struct ratbag_device *device,
-			     enum ratbag_device_capability cap);
+uint32_t
+ratbag_device_get_vendor_id(const struct ratbag_device *device);
+
+
+/**
+ * @ingroup device
+ *
+ * Returns the product ID of the device.
+ */
+uint32_t
+ratbag_device_get_product_id(const struct ratbag_device *device);
+
+/**
+ * @ingroup device
+ *
+ * Returns the product version of the device. This is a made-up number,
+ * internal to libratbag. It's purpose is to provide an extra bit of
+ * differentiation where devices use the same VID/PID for different models.
+ *
+ * Devices with identical VID/PID but different versions must be considered
+ * different devices.
+ *
+ * Usually the version will be 0.
+ */
+uint32_t
+ratbag_device_get_product_version(const struct ratbag_device *device);
 
 /**
  * @ingroup device
@@ -572,8 +587,8 @@ ratbag_profile_set_name(struct ratbag_profile *profile,
 /**
  * @ingroup profile
  *
- * Enable/disable the ratbag profile. For this to work, the device must support
- * @ref RATBAG_DEVICE_CAP_DISABLE_PROFILE.
+ * Enable/disable the ratbag profile. For this to work, the profile must support
+ * @ref RATBAG_PROFILE_CAP_DISABLE.
  *
  * @param profile A previously initialized ratbag profile
  * @param enabled Whether to enable or disable the profile
@@ -587,7 +602,7 @@ ratbag_profile_set_enabled(struct ratbag_profile *profile, bool enabled);
  * @ingroup profile
  *
  * Check whether the ratbag profile is enabled or not. For devices that don't
- * support @ref RATBAG_DEVICE_CAP_DISABLE_PROFILE the profile will always be
+ * support @ref RATBAG_PROFILE_CAP_DISABLE the profile will always be
  * set to enabled.
  *
  * @param profile A previously initialized ratbag profile
@@ -670,6 +685,61 @@ ratbag_profile_is_active(struct ratbag_profile *profile);
 enum ratbag_error_code
 ratbag_profile_set_active(struct ratbag_profile *profile);
 
+/**
+ * @ingroup profile
+ *
+ * Set the report rate in Hz for the profile.
+ *
+ * A value of 0 hz disables the mode.
+ *
+ * If the profile mode is the currently active profile,
+ * the change takes effect immediately.
+ *
+ * @param profile A previously initialized ratbag profile
+ * @param hz Set to the report rate in Hz, may be 0
+ *
+ * @return zero on success or an error code on failure
+ */
+enum ratbag_error_code
+ratbag_profile_set_report_rate(struct ratbag_profile *profile,
+			       unsigned int hz);
+
+/**
+ * @ingroup profile
+ *
+ * Get the report rate in Hz for the profile.
+ *
+ * @param profile A previously initialized ratbag profile
+ *
+ * @return The report rate for this profile in Hz
+ */
+int
+ratbag_profile_get_report_rate(struct ratbag_profile *profile);
+
+/**
+ * @ingroup profile
+ *
+ * Get the number of report rates in Hz available for this profile.
+ * The list of report rates is sorted in ascending order but may be filtered
+ * by libratbag and does not necessarily reflect all report rates supported by
+ * the physical device.
+ *
+ * This function writes at most nrates values but returns the number of
+ * report rates available on this resolution. In other words, if it returns a
+ * number larger than nrates, call it again with an array the size of the
+ * return value.
+ *
+ * @param[out] rates Set to the supported report rates in ascending order
+ * @param[in] nrates The number of elements in resolutions
+ *
+ * @return The number of valid items in rates. If the returned value
+ * is larger than nrates, the list was truncated.
+ */
+size_t
+ratbag_profile_get_report_rate_list(struct ratbag_profile *profile,
+				    unsigned int *rates,
+				    size_t nrates);
+
 /**
  * @ingroup profile
  *
@@ -898,67 +968,6 @@ ratbag_resolution_get_dpi_x(struct ratbag_resolution *resolution);
 int
 ratbag_resolution_get_dpi_y(struct ratbag_resolution *resolution);
 
-/**
- * @ingroup resolution
- *
- * Set the report rate in Hz for the resolution mode.
- *
- * A value of 0 hz disables the mode.
- *
- * If the resolution mode is the currently active mode and the profile is
- * the currently active profile, the change takes effect immediately.
- *
- * If the resolution does not have the @ref
- * RATBAG_RESOLUTION_CAP_INDIVIDUAL_REPORT_RATE capability, changing the
- * report rate on one resolution changes the report rate for all resolutions
- * in this profile.
- *
- * @param resolution A previously initialized ratbag resolution
- * @param hz Set to the report rate in Hz, may be 0
- *
- * @return zero on success or an error code on failure
- */
-enum ratbag_error_code
-ratbag_resolution_set_report_rate(struct ratbag_resolution *resolution,
-				  unsigned int hz);
-
-/**
- * @ingroup resolution
- *
- * Get the report rate in Hz for the resolution mode.
- *
- * A value of 0 hz indicates the mode is disabled.
- *
- * @param resolution A previously initialized ratbag resolution
- *
- * @return The report rate for this resolution in Hz
- */
-int
-ratbag_resolution_get_report_rate(struct ratbag_resolution *resolution);
-
-/**
- * @ingroup resolution
- *
- * Get the number of report rates in Hz available for this resolution.
- * The list of report rates is sorted in ascending order but may be filtered
- * by libratbag and does not necessarily reflect all report rates supported by
- * the physical device.
- *
- * This function writes at most nrates values but returns the number of
- * report rates available on this resolution. In other words, if it returns a
- * number larger than nrates, call it again with an array the size of the
- * return value.
- *
- * @param[out] rates Set to the supported report rates in ascending order
- * @param[in] nrates The number of elements in resolutions
- *
- * @return The number of valid items in rates. If the returned value
- * is larger than nrates, the list was truncated.
- */
-size_t
-ratbag_resolution_get_report_rate_list(struct ratbag_resolution *resolution,
-				       unsigned int *rates,
-				       size_t nrates);
 /**
  * @ingroup resolution
  *
@@ -1073,25 +1082,6 @@ ratbag_button_set_user_data(struct ratbag_button *button, void *userdata);
 void*
 ratbag_button_get_user_data(const struct ratbag_button *button);
 
-/**
- * @ingroup button
- *
- * Return the type of the physical button. This function is intended to be
- * used by configuration tools to provide a generic list of button names or
- * handles to configure devices. The type describes the physical location of
- * the button and remains constant for the lifetime of the device.
- * For example, a button of type @ref RATBAG_BUTTON_TYPE_WHEEL_CLICK may be
- * mapped to a logical middle button, but the physical description is that
- * of a wheel click.
- *
- * For the button currently mapped to this physical button, see
- * ratbag_button_get_button()
- *
- * @return The type of the button
- */
-enum ratbag_button_type
-ratbag_button_get_type(struct ratbag_button *button);
-
 /**
  * @ingroup button
  *
@@ -1193,18 +1183,6 @@ ratbag_button_get_special(struct ratbag_button *button);
  */
 struct ratbag_led *
 ratbag_profile_get_led(struct ratbag_profile *profile, unsigned int index);
-/**
- * @ingroup led
- *
- * This function returns the type for ratbag_led.
- *
- * @param led A previously initialized ratbag LED
- * @return The LED type @ref ratbag_led_type
- *
- * @see ratbag_led_set_mode
- */
-enum ratbag_led_type
-ratbag_led_get_type(struct ratbag_led *led);
 
 /**
  * @ingroup led
