@@ -115,7 +115,7 @@ main(int argc, char **argv)
 		error(1, errno, "Failed to open path %s", path);
 
 	hidpp_device_init(&base, fd);
-	dev = hidpp20_device_new(&base, 0xff);
+	dev = hidpp20_device_new(&base, 0xff, NULL, 0);
 	if (!dev)
 		error(1, 0, "Failed to open %s as a HID++ 2.0 device", path);
 
